from .reliable_communicating_node import testReliableCommunicatingNode

# Run this file using `python -m lora_p2p`

if __name__ == "__main__":
    # INTEGRATION TEST
    testReliableCommunicatingNode()